% ----------------------------------------------------------------------------
% 
% Copyright (c) 2016 Gilles Simon (gsimon[at]loria.fr)
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU Affero General Public License for more details.
% 
% This is an implementation of the method described in: 
% 
% Gilles Simon, Antoine Fond, Marie-Odile Berger 
% A Simple and Effective Method to Detect Orthogonal Vanishing Points in Uncalibrated Images of Man-Made Environments. 
% Eurographics 2016, May 2016, Lisbon, Portugal.
% 
% ----------------------------------------------------------------------------

function [imR] = orthorectify(im, H, line, s)
max_stretch = 16;
step = round(size(im, 2)/64);
width = size(im,2);
height = size(im,1);
umin = 1e10;
umax = -1e10;
vmin = 1e10;
vmax = -1e10;
imR = im;
% imBound = im;
% mask = zeros(size(imBound,1),size(imBound,2));
for i = 1:step:width
    for j = 1:step:height
        if dot(line, [i; j; 1])*(-1)^s > 0
            pix = [i,i+1;j,j+1;1,1];
            Tpix = H*pix;
            Tpix(:,1) = Tpix(:,1)/Tpix(3,1);
            Tpix(:,2) = Tpix(:,2)/Tpix(3,2);
            u = Tpix(1,1);
            v = Tpix(2,1);
            u2 = Tpix(1,2);
            v2 = Tpix(2,2);
            if abs(u2-u) < max_stretch && abs(v2-v) < max_stretch
%                 mask(j,i) = 1;
                if u < umin umin = u; else if u > umax umax = u; end; end;
                if v < vmin vmin = v; else if v > vmax vmax = v; end; end;
            end
        end
    end
end
% imBound(:,:,2)=double(imBound(:,:,2)).*double(1-mask);
% imBound(:,:,3)=double(imBound(:,:,3)).*double(1-mask);
heightT = floor(vmax-vmin);
widthT = floor(umax-umin);
if heightT > 0 && widthT > 0
    ref = imref2d([heightT,widthT],[umin,umax],[vmin,vmax]);
    tform = projective2d(H');
    imR = imwarp(im,tform,'OutputView',ref);
end
end
